;debug macro
macro outhb number, buffer{
  push esi ecx eax ebx
  mov  ecx, number
  mov  ebx, 2
  mov  esi, buffer
  call hex2ascii
  pop  ebx eax ecx esi
}

macro show size, buffer{
  push esi ecx eax ebx
  mov  ebx, size
  mov  esi, buffer
  call	ascii_out
  pop  ebx eax ecx esi
}


;#########################################################################

;compile with latest Fasm
;write at the beginning of floppy
;has to be 1st floppy drive (Drive A: in Windows) to boot

org 7c00h
use16
boot_sector:
  cli

  ;clear screen
  mov  ax, 3
  int  10h

;  mov  ax, 7e00h
;  mov  ss, ax
;  mov  sp, ax

  ;load next several sectors
  mov  ax, 020ah   ; AH = Function, AL = Number of sectors to read (1-128)
  mov  cx, 2	   ; CH = Cylinder number (0-1023), CL = Sector number (1-17)
  xor  dx, dx	   ; DH = Head number (0-15), DL = Drive number (0-A:, 1-B:)
  mov  bx, boot_sector+512    ; destination pointer
  int  13h
  jc   .error	   ; jump if read error

  ;floppy off
  mov  dx,3f2h
  xor  al,al
  out  dx,al

  ;no ints
  cli
  in   al, 70h
  or   al, 80h
  out  70h, al

  ;enable A20 line
  in   al, 92h
  or   al, 2
  out  92h, al

;  ;set PIC timer to trigger 1/100 s
;  mov  al,0x34    ; 0x34=00110100
;  out  0x43,al    ; 100GHz, 1cs(10ms)
;  mov  al,0x9b    ; lsb 1193180 / 11931 = 2e9b
;  out  0x40,al
;  mov  al,0x2e    ; msb
;  out  0x40,al

  lgdt [cs:GDT.reg]	      ; load GDT
  mov  eax, cr0
  or   al, 1		      ; set PE flag
  ;or   eax, (1 shl 29) or (1 shl 30) ;no caching
  mov  cr0, eax 	      ; enable protected mode
  jmp  8h:PMode32   ; jump to protected mode segment

  ;notify about error
.error:
  mov  ax, 0b800h
  mov  ds, ax
  mov  ax, 0404h
  mov  [ds:0], ax ;red diamond on screen
  jmp  $

;#########################################################################

align 8
GDT:									;selector
  dq 0
;           [ LIMIT    |      BASE    | PDLSTYPE   GD0ALIMT | BASE ]
	 db  0FFh, 0FFh, 00h, 00h, 00h, 10011010b, 11001111b, 00	; 8h
	 db  0FFh, 0FFh, 00h, 00h, 00h, 10010010b, 11001111b, 00	; 10h
  .reg:
  .limit  dw $-GDT-1
	  dd GDT

rb 510-$+boot_sector
dw 0aa55h

;#########################################################################

;#GP will be executed if interrupt that has "dq 0"(vector not set up) fires
IDT:   ; IRQ #
  dq 0 ; 0
  dq 0 ; 1
  dq 0 ; 2
  dq 0 ; 3
  dq 0 ; 4
  dq 0 ; 5
  dq 0 ; 6
  dq 0 ; 7
  dq 0 ; 8
  dq 0 ; 9
  dq 0 ; 10
  dq 0 ; 11
  dq 0 ; 12
  dw int_GP, 08h, 1000111000000000b, 0	 ; 13 #GP, General Protection Fault
  dq 0 ; 14
  dq 0 ; 15
  dq 0 ; 16
  dq 0 ; 17
  dq 0 ; 18
  dq 0 ; 19
  dq 0 ; 20
  dq 0 ; 21
  dq 0 ; 22
  dq 0 ; 23
  dq 0 ; 24
  dq 0 ; 25
  dq 0 ; 26
  dq 0 ; 27
  dq 0 ; 28
  dq 0 ; 29
  dq 0 ; 30
  dq 0 ; 31
		     ;8e00h
  dw int_timer, 08h, 1000111000000000b, 0   ; 32 timer
  dw int_kbd, 08h, 1000111000000000b, 0   ; 33 keyboard
  dq 0	 ; 34
  dq 0	 ; 35
  dq 0	 ; 36
  dq 0	 ; 37
  dq 0	 ; 38
  dq 0	 ; 39

  dq 0	 ; 40
  dq 0	 ; 41
  dq 0	 ; 42
  dq 0	 ; 43
  dq 0	 ; 44
  dq 0	 ; 45
  dq 0	 ; 46
  dq 0	 ; 47

  .reg:
  .limit  dw $-IDT-1
  .addr   dd IDT

cursor dd 0b8000h+162
buff1 db 30 dup(0)

;#########################################################################

use32
PMode32:
  xor  ecx, ecx
  mov  eax, 10h    ; load 4 GB data descriptor
  mov  ds,ax	   ; to all segment registers
  mov  ss,ax
  mov  es,ax
  mov  fs,cx
  mov  gs,cx

  mov word [0b8000h], 0204h ;green diamond on screen

  ;from Fasm website
  mov	  al,10001b		  ; begin PIC 1 initialization
  out	  20h,al
  mov	  al,10001b		  ; begin PIC 2 initialization
  out	  0A0h,al
  mov	  al,33 		  ; IRQ 0-7: IRQs 32-39
  out	  21h,al
  mov	  al,41 		  ; IRQ 8-15: IRQs 40-47
  out	  0A1h,al
  mov	  al,100b		  ; slave connected to IRQ2
  out	  21h,al
  mov	  al,2
  out	  0A1h,al
  mov	  al,1			  ; Intel environment, manual EOI
  out	  21h,al
  out	  0A1h,al

;  in      al,21h
;  mov     al,11111100b            ; enable only clock and keyboard IRQ
;  out     21h,al
;  in      al,0A1h
;  mov     al,11111111b
;  out     0A1h,al

  lidt [IDT.reg]

  mov word [0b8000h+2], 0504h ;pink diamond on screen
;  jmp  $

  in   al, 70h
  and  al, 7Fh
  out  70h, al
  sti

  mov word [0b8000h+4], 0104h ;blue diamond on screen

  jmp  $

;#########################################################################

int_kbd:
  push eax ebx ecx edx esi edi
  pushfd
  ;mov word [0b8000h+40], 0404h ;red diamond on screen

  xor  eax, eax
  in   al, 60h

  outhb eax, buff1
  show	2, buff1

  popfd
  pop  edi esi edx ecx ebx eax
  jmp  EOI

;#########################################################################

int_timer:
  add word [0b8000h+6], 1
  jmp  EOI

;#########################################################################

int_GP:
  pop  eax
  mov word [0b8000h+80*6], 0104h ;blue diamond on screen on 3rd line
  iretd

;#########################################################################

EOI:
  push eax
  mov  eax, 20h
  out  020h, al   ; Master
  out  0a0h, al   ; Slave
  pop  eax
iretd

;Restore_Regs:
;  ;--debug---display int name & count---
;  push rax
;  mov  esi, errors
;  add  byte [esi+edi], 1
;  cmp  byte [esi+edi], '9'
;  jbe  .ok_err_number
;  mov  byte [esi+edi], '0'
;.ok_err_number:
;  mov  edx, [cursor2]
;  mov  [cursor2], 3840
;  mov  ebx, errors_sz
;  call ascii_out
;  mov  [cursor2], edx
;  pop  rax





hex2ascii:
  mov  eax, ecx
  shr  ecx, 4
  and  eax, 0fh
  add  eax, '0'
  cmp  al, '9'
  jle  .ok  ;DAS missing!
  add  eax, "A"-'9'-1
.ok:
  sub  ebx, 1
  mov  [esi+ebx], al
  jnz  hex2ascii
ret


ascii_out:
  mov  ecx, [cursor]
  lea  eax, [ebx*2+2]
  ;add  ecx, 0b8000h
  add  [cursor], eax
.out_bait:
  xor  eax, eax
  sub  ebx, 1
  mov  al, byte [esi+ebx]
  mov  [ecx+ebx*2], al
  jnz  .out_bait
.out_dword:
ret
